/**
 * Convert a dataset in dat format to a asc format as specified by ARtool
 *
 * Copyright 2014 Matteo Riondato <matteo@cs.brown.edu>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */

#include <cerrno>
#include <climits>
#include <cstdio>
#include <iostream>
#include <map>
#include <set>
#include <sstream>

#include "util.h"

using namespace std;

int main(int argc, char **argv) {
    if (argc != 2) {
		cerr << argv[0] << ": convert a dataset from dat to asc format" << endl;
        cerr << "USAGE: " << argv[0] << " dataset" << endl;
        return 1;
    }

	FILE *ds_FILE = fopen(argv[1], "r");
	if (ds_FILE == NULL) {
		perror("Error opening dataset file");
		return errno;
	}

    char line[TRANSACTION_LINE_MAXLEN];
    set<unsigned int> items;
    while (get_next_line(line, sizeof(line), ds_FILE)) {
        set<unsigned int> transaction = transaction_to_set(line);
        items.insert(transaction.begin(), transaction.end());
    }
    rewind(ds_FILE);

    map<unsigned int, unsigned int> items_map;
    int prev = 0;
    unsigned int curr = 0;
    for (unsigned int item : items) {
        if (prev < ((int) item - 1)) {
            cerr << "Missing items from " << prev + 1 << " to " << ((int) item) - 1 << endl;;
        }
        ++curr;
        items_map[item] = curr;
        cout << curr << SPACE << item << endl;
        prev = (int) item;
    }

    cout << "BEGIN_DATA" << endl;
    while (get_next_line(line, sizeof(line), ds_FILE)) {
        set<unsigned int> transaction = transaction_to_set(line);
        stringstream stream;
        for (unsigned int item : transaction) {
            stream << items_map[item] << SPACE;
        }
        stream.unget(); // remove last space
        cout << stream.str() << endl;
    }
    cout << "END_DATA" << endl;

    fclose(ds_FILE);

    return 0;
}

